package cn.fw.dalaran.rpc.ehr;

import cn.fw.dalaran.common.utils.PublicUtil;
import cn.fw.dalaran.rpc.dto.StaffBaseInfoRpc;
import cn.fw.dalaran.rpc.dto.StaffInfoRpc;
import cn.fw.dalaran.rpc.dto.StaffShopSelectRpc;
import cn.fw.data.base.domain.common.Message;
import cn.fw.ehr.sdk.api.StaffApi;
import cn.fw.ehr.sdk.api.dto.ManagerScopeCheckDto;
import cn.fw.ehr.sdk.api.result.StaffBaseInfo;
import cn.fw.ehr.sdk.api.result.StaffInfo;
import cn.fw.ehr.sdk.api.result.StaffShopSelectVo;
import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2021/9/17 10:01
 * @Description 远程调用人事系统
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class EhrRpc {

    private final StaffApi staffApi;

    /**
     * @param staffIds    需要收到公示的人员的id集合
     * @param publisherId 发布公示的人员的id
     * @return 每个员工的是否在该发布者的管理范围的封装信息
     */
    public List<StaffShopSelectRpc> checkBeyondManage(Set<Long> staffIds, Long publisherId) {
        try {
            ManagerScopeCheckDto managerScopeCheckDto = new ManagerScopeCheckDto();// 准备远程调用人事系统, 封装请求需要的参数
            managerScopeCheckDto.setStaffIdList(new ArrayList<>(staffIds));// 设置staffIdList参数
            managerScopeCheckDto.setStaffId(publisherId);// 设置发布公示的职工的id
            Message<List<StaffShopSelectVo>> StaffShopSelectList = staffApi.checkManagerScope(managerScopeCheckDto);
            PublicUtil.assertTrue(StaffShopSelectList.isSuccess(), StaffShopSelectList.getResult());
            log.info("远程调用人事服务 staffApi.checkManagerScope 成功, 传入参数为 managerScopeCheckDto: {} 结果为: {}", managerScopeCheckDto, JSON.toJSONString(StaffShopSelectList));
            return PublicUtil.copyList(StaffShopSelectList.getData(), StaffShopSelectRpc.class);
        } catch (Exception e) {
            log.error("远程调用人事服务 staffApi.checkManagerScope 发生异常, 传入参数为 staffIds: {}, publisherId: {} 失败!", staffIds, publisherId);
        }
        return null;
    }

    /**
     * 根据集团id, 获取集团所有人员的信息
     *
     * @param groupId 集团id
     * @return 集团所有员工的信息
     */
    public List<StaffInfoRpc> getGroupStaffList(Long groupId) {
        try {
            Message<List<StaffInfo>> groupStaffList = staffApi.getGroupStaffList(groupId);
            PublicUtil.assertTrue(groupStaffList.isSuccess(), groupStaffList.getResult());
            log.info("远程调用人事服务 staffApi.getGroupStaffList 成功, 传入参数为 groupId: {} 结果为: {}", groupId, JSON.toJSONString(groupStaffList));
            return PublicUtil.copyList(groupStaffList.getData(), StaffInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getGroupStaffList 发生异常, 传入参数为 groupId: {} 异常!", groupId);
        }
        return null;
    }

    /**
     * 根据职工id获取职工基础信息
     *
     * @param staffId 职工的id
     * @return 该职工基础信息
     */
    public StaffBaseInfoRpc getStaffBaseInfo(Long staffId) {
        try {
            Message<StaffBaseInfo> staffBaseInfo = staffApi.getStaffBaseInfo(staffId);
            log.info("远程调用 staffApi.getStaffBaseInfo 成功, 传入参数为 staffId: {} 结果为: {}", staffId, JSON.toJSONString(staffBaseInfo));
            PublicUtil.assertTrue(staffBaseInfo.isSuccess(), staffBaseInfo.getResult());
            return PublicUtil.copy(staffBaseInfo.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getStaffBaseInfo 发生异常, 传入参数为 staffId: {} 异常!", staffId);
        }
        return null;
    }

    /**
     * 根据职工id集合批量获取职工基础信息
     *
     * @param staffIds 职工的id的集合
     * @return 职工的基础信息集合
     */
    public List<StaffBaseInfoRpc> getStaffInfoBatch(List<Long> staffIds) {
        try {
            Message<List<StaffBaseInfo>> batchStaffBaseInfoList = staffApi.getBatchStaffBaseInfoList(staffIds);
            log.info("远程调用 staffApi.getBatchStaffBaseInfoList 成功, 传入参数为 staffIds: {} 结果为: {}", staffIds, JSON.toJSONString(batchStaffBaseInfoList));
            PublicUtil.assertTrue(batchStaffBaseInfoList.isSuccess(), batchStaffBaseInfoList.getResult());
            return PublicUtil.copyList(batchStaffBaseInfoList.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getBatchStaffBaseInfoList 发生异常, 传入参数为 staffId: {} 异常!", staffIds);
        }
        return null;
    }

    /**
     * 根据职工id获取职工信息
     *
     * @param staffId 职工的id
     * @return 该职工其他信息
     */
    public StaffInfoRpc getStaffOtherInfo(Long staffId) {
        try {
            Message<StaffInfo> staffInfo = staffApi.getStaffInfoById(staffId);
            log.info("远程调用 staffApi.getStaffInfoById 成功, 传入参数为 staffId: {} 结果为: {}", staffId, JSON.toJSONString(staffInfo));
            PublicUtil.assertTrue(staffInfo.isSuccess(), staffInfo.getResult());
            return PublicUtil.copy(staffInfo.getData(), StaffInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getStaffInfoById 发生异常, 传入参数为 staffId: {} 异常!", staffId);
        }
        return null;
    }

    /**
     * 根据 门店id 获取职工信息
     *
     * @param shopId 门店id
     * @return 该门店下所有员工信息
     */
    public List<StaffBaseInfoRpc> getShopStaffList(Long shopId) {
        try {
            Message<List<StaffBaseInfo>> postShopStaffList = staffApi.getPostShopStaffList(null, shopId);
            log.info("远程调用 staffApi.getPostShopStaffList 成功, 传入参数为 shopId: {} 结果为: {}", shopId, JSON.toJSONString(postShopStaffList));
            PublicUtil.assertTrue(postShopStaffList.isSuccess(), postShopStaffList.getResult());
            return PublicUtil.copyList(postShopStaffList.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getPostShopStaffList 发生异常, 传入参数为 shopId: {} 异常!", shopId);
        }
        return null;
    }

    /**
     * 根据 岗位id 获取职工信息
     *
     * @param postId 门店id
     * @return 该岗位的所有员工信息
     */
    public List<StaffBaseInfoRpc> getPostStaffList(Long postId) {
        try {
            Message<List<StaffBaseInfo>> postShopStaffList = staffApi.getPostShopStaffList(postId, null);
            log.info("远程调用 staffApi.getPostShopStaffList 成功, 传入参数为 postId: {} 结果为: {}", postId, JSON.toJSONString(postShopStaffList));
            PublicUtil.assertTrue(postShopStaffList.isSuccess(), postShopStaffList.getResult());
            return PublicUtil.copyList(postShopStaffList.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getPostShopStaffList 发生异常, 传入参数为 postId: {} 异常!", postId);
        }
        return null;
    }

    /**
     * 根据 岗位id+门店id 获取职工信息
     *
     * @param postId 岗位id
     * @param shopId 门店id
     * @return 该 岗位+门店 的所有员工信息
     */
    public List<StaffBaseInfoRpc> getPostShopStaffList(Long postId, Long shopId) {
        try {
            Message<List<StaffBaseInfo>> postShopStaffList = staffApi.getPostShopStaffList(postId, shopId);
            log.info("远程调用 staffApi.getPostShopStaffList 成功, 传入参数为 postId: {} shopId: {} 结果为: {}", postId, shopId, JSON.toJSONString(postShopStaffList));
            PublicUtil.assertTrue(postShopStaffList.isSuccess(), postShopStaffList.getResult());
            return PublicUtil.copyList(postShopStaffList.getData(), StaffBaseInfoRpc.class);
        } catch (Exception e) {
            log.error("调用人事系统 staffApi.getPostShopStaffList 发生异常, 传入参数为 postId: {} shopId: {} 异常!", postId, shopId);
        }
        return null;
    }

}
