package cn.fw.dalaran.service.data.impl;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.dalaran.common.ConfigProperties;
import cn.fw.dalaran.common.constants.Constants;
import cn.fw.dalaran.common.exception.BusinessException;
import cn.fw.dalaran.common.utils.DateUtil;
import cn.fw.dalaran.common.utils.ImageUtils;
import cn.fw.dalaran.dao.ActivityThemeDao;
import cn.fw.dalaran.domain.db.ActivityTheme;
import cn.fw.dalaran.domain.db.ConfigGroup;
import cn.fw.dalaran.domain.db.ThemeFile;
import cn.fw.dalaran.domain.enums.BizTypeEnum;
import cn.fw.dalaran.domain.enums.FileTypeEnum;
import cn.fw.dalaran.domain.enums.ValidLiveEnum;
import cn.fw.dalaran.domain.enums.ValidVideoEnum;
import cn.fw.dalaran.domain.param.ActivityThemePageParams;
import cn.fw.dalaran.domain.param.AddThemeCoverParams;
import cn.fw.dalaran.domain.param.AddThemeTopicParams;
import cn.fw.dalaran.domain.vo.ActivityThemeVo;
import cn.fw.dalaran.domain.vo.FileDesc;
import cn.fw.dalaran.domain.vo.ValidConfigNewVo;
import cn.fw.dalaran.rpc.erp.ErpTaskRpc;
import cn.fw.dalaran.rpc.erp.dto.TaskSetReq;
import cn.fw.dalaran.service.Common;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.ConfigGroupService;
import cn.fw.dalaran.service.data.ThemeFileService;
import cn.fw.dalaran.service.data.ValidConfigNewService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.io.File;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.LockSupport;
import java.util.stream.Collectors;

/**
 * @author wmy3969
 * @version 1.0
 * @date 2022/1/8 17:30
 * @Description
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class ActivityThemeServiceImpl extends ServiceImpl<ActivityThemeDao, ActivityTheme> implements ActivityThemeService {

    private final ConfigProperties configProperties;
    private final ActivityThemeDao activityThemeDao;
    private final ConfigGroupService configGroupService;
    private final ThemeFileService themeFileService;
    private final ValidConfigNewService validConfigNewService;
    private final ErpTaskRpc erpTaskRpc;
    private final Common common;

    /**
     * 校验该活动主题生效时间段是否与已有的活动主题时间段重叠
     *
     * @param vo 需要保存的活动
     * @return 是否有重复
     */
    public boolean validThemeDateDuplicate(ActivityThemeVo vo) {
        final ConfigGroup configGroup = configGroupService.getById(vo.getConfigGroupId());
        final List<ActivityTheme> themeList = this.lambdaQuery()
                .ge(ActivityTheme::getGroupId, configGroup.getGroupId())
                .gt(ActivityTheme::getEndTime, new Date())
                .list();
        return themeList.stream()
                .anyMatch(item -> validDateOverlap(item.getStartTime(), item.getEndTime(), vo.getStartTime(), vo.getEndTime()) &&// 验证时间重叠
                        /*(Objects.equals(item.getAllShop(), 1) || vo.getAllShop() || validShopOverlap(Arrays.stream(item.getShopIds().split(",")).map(Long::valueOf).collect(Collectors.toList()), vo.getShopIds()))// 验证门店重叠*/
                        (Objects.equals(item.getAllShop(), 1) || configGroup.getAllShop() > 0 ||
                                validShopOverlap(Arrays.stream(item.getShopIds().split(","))
                                        .map(Long::valueOf)
                                        .collect(Collectors.toList()), Arrays.stream(configGroup.getShopIds().split(","))
                                        .map(Long::valueOf)
                                        .collect(Collectors.toList())
                                )
                        )// 验证门店重叠
                );
    }

    /**
     * 验证门店是否重合
     *
     * @param shopIds1
     * @param shopIds2
     * @return
     */
    private boolean validShopOverlap(List<Long> shopIds1, List<Long> shopIds2) {
        if (Objects.equals(shopIds1.get(0), -1L) || Objects.equals(shopIds2.get(0), -1L)) {
            return true;
        } else {
            shopIds1.retainAll(shopIds2);
            return !shopIds1.isEmpty();
        }
    }

    /**
     * 校验时间有无重叠
     *
     * @param startDate1 开始时间1
     * @param endDate1   结束时间1
     * @param startDate2 开始时间2
     * @param endDate2   结束时间2
     * @return 两个时间区间是否有重叠
     */
    public boolean validDateOverlap(Date startDate1, Date endDate1, Date startDate2, Date endDate2) {
        if (startDate1.after(endDate1) || startDate2.after(endDate2))
            throw new BusinessException("开始时间必须在结束时间之前");
        return !endDate2.before(startDate1) && !startDate2.after(endDate1) && !endDate1.before(startDate2) && !startDate1.after(endDate2);
    }

    /**
     * 保存主题
     *
     * @param themeVo 参数
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean save(ActivityThemeVo themeVo) {
        ConfigGroup configGroup = configGroupService.getById(themeVo.getConfigGroupId());
        ActivityTheme activityTheme = ActivityThemeVo.toDB(themeVo);
        this.assembleConfigJson(themeVo.getConfigGroupId(), activityTheme);
        activityTheme.setConfigGroupId(configGroup.getId());
        activityTheme.setGroupId(configGroup.getGroupId());
        activityTheme.setAllShop(configGroup.getAllShop());
        activityTheme.setShopIds(configGroup.getShopIds());
        activityTheme.setShopNames(configGroup.getShopNames());
        activityTheme.setLogicDel(0);
        boolean result = this.save(activityTheme);
        if (result) {
            erpTaskRpc.setTask(TaskSetReq.<Long>builder()
                    .executeTime(new Date(activityTheme.getStartTime().getTime() + 24 * 3600 * 1000L))
                    .system(Constants.APPLICATION_NAME)
                    .param(activityTheme.getId())
                    .tag(Constants.INIT_ACCOUNT_STATUS)
                    .build()
            );
            this.saveFiles(activityTheme.getId(), themeVo.getAllFileDesc());
            this.downloadAndConvertCover(activityTheme, themeVo.getAllFileDesc());
        }
        return result;
    }

    /**
     * 增加话题
     *
     * @param params 增加话题带参
     * @return
     */
    @Override
    public boolean addTopic(AddThemeTopicParams params) {
        final ActivityTheme activityTheme = AddThemeTopicParams.toDB(params);
        final Long themeId = params.getThemeId();
        final ActivityTheme theme = this.getById(themeId);
        if (Objects.isNull(theme))
            throw new BusinessException(String.format("不存在id为: %s的活动主题", themeId));
        List<String> newTopic = params.getTopic().stream()
                .map(item -> {
                    String replace = item.replace("#", "");
                    return "#" + replace;
                })
                .collect(Collectors.toList());
        List<String> oldTopic = Arrays.asList(theme.getTopic().split(",").clone());
        ArrayList<String> result = new ArrayList<>(newTopic);
        result.retainAll(oldTopic);
        if (this.themeHaveStopCatchData(theme.getEndTime(), 0))
            throw new BusinessException("活动已经结束数据抓取, 不能再增加话题");
        if (result.size() < oldTopic.size())
            throw new BusinessException("不允许删除原有话题");
        if (Objects.equals(newTopic.size(), oldTopic.size()))
            throw new BusinessException("未进行任何添加操作");
        return this.updateById(activityTheme);
    }

    /**
     * 增加封面
     *
     * @param params 增加封面带参
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addCover(AddThemeCoverParams params) {
        Long themeId = params.getThemeId();
        List<FileDesc> fileList = params.getCovers()
                .stream()
                .filter(item -> Objects.isNull(item.getId()))
                .peek(item -> item.setType(FileTypeEnum.THEME_COVER.getValue()))
                .collect(Collectors.toList());
        ActivityTheme theme = this.getById(themeId);
        if (this.themeHaveStopCatchData(theme.getEndTime(), 0))
            throw new BusinessException("活动已经结束数据抓取, 不能再增加封面");
        this.saveFiles(themeId, fileList);
        this.downloadAndConvertCover(theme, fileList);
        return true;
    }

    /**
     * 查询集团话题列表
     *
     * @param groupId       集团id
     * @param configGroupId 所属配置组id
     * @return
     */
    @Override
    public List<ActivityThemeVo> list(Long groupId, Long configGroupId) {
        return this.lambdaQuery()
                .eq(ActivityTheme::getGroupId, groupId)
                .eq(ActivityTheme::getConfigGroupId, configGroupId)
                .orderByDesc(ActivityTheme::getId)
                .list()
                .stream()
                .map(ActivityTheme::toVO)
                .peek(this::peekActivityThemeVo)
                .collect(Collectors.toList());
    }

    /**
     * 分页查询活动主题列表
     *
     * @param groupId    集团id
     * @param pageParams 分页查询参数
     * @return
     */
    @Override
    public AppPage<ActivityThemeVo> pageList(Long groupId, ActivityThemePageParams pageParams) {
        Page<ActivityTheme> dbPageList = new Page<>(pageParams.getCurrent(), pageParams.getPageSize());
        dbPageList = this.page(dbPageList, Wrappers.<ActivityTheme>lambdaQuery()
                .eq(ActivityTheme::getGroupId, groupId)
                .eq(ActivityTheme::getConfigGroupId, pageParams.getConfigGroupId())
                .orderByDesc(ActivityTheme::getId));
        AppPage<ActivityThemeVo> pageList = AppPage.empty(pageParams);
        pageList.setTotal(dbPageList.getTotal());
        List<ActivityTheme> dbList = dbPageList.getRecords();
        if (CollectionUtils.isEmpty(dbList)) {
            return pageList;
        }
        pageList.setData(dbList.stream()
                .map(ActivityTheme::toVO)
                .peek(this::peekActivityThemeVo)
                .collect(Collectors.toList())
        );
        return pageList;
    }

    /**
     * 查询活动列表(指定月)
     *
     * @param user      当前登录用户
     * @param timeStamp 指定月之内的时间戳
     * @return 符合条件的活动列表
     */
    @Override
    public List<ActivityThemeVo> listByMonth(LoginAuthBean user, Long timeStamp) {
        Date date = new Date(timeStamp);// 时间戳指定时间
        Date startTime = DateUtil.getMonthFirstDay(DateUtil.getBeginInTime(date));
        Date endTime = DateUtil.getMonthEndDay(DateUtil.getEndInTime(date));
        Date yesterdayNow = new Date(System.currentTimeMillis() - 24 * 3600 * 1000L);
        if (endTime.compareTo(yesterdayNow) >= 0)
            endTime = yesterdayNow;
        Date finalEndTime = endTime;
        return this.lambdaQuery()
                .eq(ActivityTheme::getGroupId, user.getGroupId())
                .le(ActivityTheme::getStartTime, endTime)
                .ge(ActivityTheme::getEndTime, startTime)
                .list()
                .stream()
                .filter(item -> this.validDateOverlap(item.getStartTime(), item.getEndTime(), startTime, finalEndTime))
                .map(ActivityTheme::toVO)
                .peek(this::peekActivityThemeVo)
                .sorted(Comparator.comparing(ActivityThemeVo::getStartTime))
                .collect(Collectors.toList())
                ;
    }

    /**
     * 修改活动主题
     *
     * @param param 修改的参数封装
     */
    @Override
    public boolean alertTheme(ActivityTheme param) {
        return activityThemeDao.alertTheme(param);
    }

    /**
     * 组装配置
     *
     * @param configGroupId 配置组id
     * @param activityTheme 活动主题实体
     */
    private void assembleConfigJson(Long configGroupId, ActivityTheme activityTheme) {
        activityTheme.setValidConfigJson(this.getValidConfigJson(configGroupId));
    }

    /**
     * 获取配置组对应有效性配置
     *
     * @param configGroupId 配置组id
     */
    @Override
    public String getValidConfigJson(Long configGroupId) {
        final List<ValidConfigNewVo.singleVo> singleVoList = validConfigNewService.queryList(configGroupId, BizTypeEnum.VIDEO.getValue()).getDetails();
        singleVoList.addAll(validConfigNewService.queryList(configGroupId, BizTypeEnum.LIVE.getValue()).getDetails());
        final List<JSONObject> collect = singleVoList.stream().map(item -> {
            final JSONObject obj = new JSONObject();
            final Integer type = item.getType();
            final Integer indexType = item.getIndexType();
            obj.put("type", type);
            obj.put("indexType", indexType);
            obj.put("indexTypeName", Objects.equals(type, BizTypeEnum.VIDEO.getValue()) ? ValidVideoEnum.getNameByVale(indexType) : ValidLiveEnum.getNameByVale(indexType));
            obj.put("minValue", item.getMinValue());
            obj.put("valid", item.getValid());
            return obj;
        }).collect(Collectors.toList());
        return JSON.toJSONString(collect);
    }

    /**
     * 保存主题封面图信息
     *
     * @param themeId     主题id
     * @param allFileDesc 文件信息
     * @return
     */
    private boolean saveFiles(Long themeId, List<FileDesc> allFileDesc) {
        // 获取主题封面图信息
        List<ThemeFile> fileList = Optional.ofNullable(allFileDesc)
                .orElse(Lists.newArrayList())
                .stream()
                .map(item -> ThemeFile.builder()
                        .themeId(themeId)
                        .fileName(item.getFileName())
                        .fileId(item.getFid())
                        .fileType(item.getFileType())
                        .type(item.getType())
                        .build())
                .collect(Collectors.toList());
        if (!fileList.isEmpty())
            return themeFileService.saveBatch(fileList);
        return true;
    }

    /**
     * 下载封面图并转换分辨率, 放入系统临时文件夹
     *
     * @param allFileDesc   所有文件信息
     * @param activityTheme 活动实体
     * @return
     */
    private void downloadAndConvertCover(ActivityTheme activityTheme, List<FileDesc> allFileDesc) {
        String tempDir = common.getActivityThemeCoverDir();
        allFileDesc.forEach(item -> {
            try {
                String fileId = item.getFid();
                File downloadFile = ImageUtils.convertFileByUrl(configProperties.getShow() + fileId, fileId);
                ImageUtils.modifyResolution1(downloadFile.getPath(),
                        tempDir + "activityTheme" + File.separator + activityTheme.getId() /*+ "#" + activityTheme.getTheme()*/ + File.separator + "settingCover",
                        "fw_theme_cover_" + fileId,
                        512, 512);
                boolean result = downloadFile.delete();
            } catch (Exception e) {
                log.error("下载转换封面图失败", e);
            }
        });
    }

    /**
     * 活动主题数据封面图处理
     */
    public static void main(String[] args) {
        int themeId = 85;
        String tempDir = System.getProperty("java.io.tmpdir");// 获取当前环境临时文件夹路径
        Arrays.asList("26b284da426c41cda324f3a11568991f",
                "417ac7dc65a641aea4153721742bcc71",
                "57610873d190488baa29449d094f31db",
                "5c3dc2e118ed436485185d9e4ee55bff",
                "5e33ad337aa04b44988df9498c5ddd43",
                "5f2caa0b050d4d63b6288ddb64c1748a",
                "5f3650d43fba461588396db5d07ef5c6",
                "71ff5e4eb1e8487c82b2f0f1e45fba3e",
                "721292bc2515482cb49f841a2ec40b1a",
                "78fe2f5e566047c7b6b41d227b77858c",
                "81316e2bb88648119194dfabf986439e",
                "924d99014f7a4223b86a67e669d34b61",
                "a0af2e55e03a4c1c852a6c0e081bc49f",
                "a804bee84e6d4084a123d729abe789fb",
                "b0c6f5ba7e7d4300aea5b1312afb7f55",
                "d478442fe3ee431b916d0f74ce72cf66",
                "d4e2df114e0a42ec8eb3ff3b36a6e168",
                "dd7f88410d124f8b8d15d2f142005ba0",
                "e73c04b8011a45ff8162cfb0750a4aef",
                "ea8d6dcad8d94be4b7234c076fe2363d",
                "ee484477b9fd44ca8c096360312ca212")
                .forEach(item -> {
                    try {
                        File downloadFile = ImageUtils.convertFileByUrl("https://gate.feewee.cn/file/show?fid=" + item, item);
                        ImageUtils.modifyResolution(downloadFile.getPath(),
                                tempDir + "activityTheme" + File.separator + themeId /*+ "#" + activityTheme.getTheme()*/ + File.separator + "settingCover",
                                "fw_theme_cover_" + item,
                                512, 512);
                        boolean result = downloadFile.delete();
                    } catch (Exception e) {
                        log.error("下载转换封面图失败", e);
                    }
                });
    }

    /**
     * 监听器, 收到消息后执行更新主题信息
     *
     * @param msg 消息
     */
    @Async
    @EventListener(String.class)
    public void updateActivityThemeValidConfigJsonMsg(String msg) {
        log.info(Thread.currentThread().getName() + " spring监听器在ActivityThemeServiceImpl.updateActivityThemeMsg()方法上成功收到消息: " + msg);
        final String[] split = msg.split("#");
        if (split.length < 2)
            return;
        LockSupport.parkNanos(TimeUnit.SECONDS.toNanos(1));
        final Long configGroupId = Long.valueOf(split[1]);
        final Date yesterday = new Date(System.currentTimeMillis() - 24 * 3600 * 1000L);
        final String validConfigJson = this.getValidConfigJson(configGroupId);
        this.lambdaUpdate()
                .set(ActivityTheme::getValidConfigJson, validConfigJson)
                .eq(ActivityTheme::getConfigGroupId, configGroupId)
                .and(wrapper -> wrapper
                        .lt(ActivityTheme::getStartTime, yesterday)
                        .gt(ActivityTheme::getEndTime, yesterday)
                        .or()
                        .gt(ActivityTheme::getStartTime, yesterday)
                )
                .update();
    }

    /**
     * 验证主题是否结束数据抓取
     *
     * @param themeEndTime 主题结束时间
     * @param delayDays    延期天数
     * @return 主题是否结束数据抓取
     */
    @Override
    public boolean themeHaveStopCatchData(Date themeEndTime, int delayDays) {
        long timeSub = System.currentTimeMillis() - themeEndTime.getTime();
        return timeSub > (1 + delayDays) * 24 * 3600 * 1000L + 1000;
    }

    /**
     * 处理活动主题, 赋其他值
     *
     * @param item 待处理的活动主题
     */
    private void peekActivityThemeVo(ActivityThemeVo item) {
        item.setCanAlter(!this.themeHaveStopCatchData(item.getEndTime(), 0));// 标识是否结束数据抓取
        item.setAllFileDesc(Optional.ofNullable(
                themeFileService.lambdaQuery()
                        .eq(ThemeFile::getThemeId, item.getId())
                        .eq(ThemeFile::getType, FileTypeEnum.THEME_COVER.getValue())
                        .list())
                .orElse(new ArrayList<>())
                .stream()
                .map(ThemeFile::toVO)
                .collect(Collectors.toList())
        );// 设置文件信息
    }

    /**
     * 获取已经逻辑删除的主题列表
     *
     * @return 已经逻辑删除的主题列表
     */
    @Override
    public List<ActivityTheme> getHasDelActivityTheme() {
        return activityThemeDao.getHasDelActivityTheme();
    }

}
